using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Xml;
using Syncfusion.Runtime.Serialization;
using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Tools.XPMenus;

namespace Baby3dx
{
    public partial class Explorer : Form
    {
       public static Explorer Instance;
       public TabbedMDIManager mdiManager = new TabbedMDIManager();

       public Explorer()
       {
          InitializeComponent();
          mainFrameBarManager1.RegisterMdiChildTypes(new Type[] { typeof(View) });
          
          mdiManager.AttachToMdiContainer(this);
          Instance = this;
       }

       private void biViewNew_Click(object sender, EventArgs e)
       {
          View sf = new View();
          sf.MdiParent = this;
          sf.Show();

          //sf.Closed += delegate { MessageBox.Show("closed."); };
       }

       private static BarItem FindByText(IList items, string text)
       {
          foreach (BarItem b in items)
          {
             // ignore the "&" symbol
             if (b.Text.Replace("&", "") == text.Replace("&", "")) return b;
          }
          return null;
       }

       private static BarItem FindByText1(BarManager bm, IList items, string text)
       {
          List<BarItem> mergedItems = new List<BarItem>();
          foreach (BarItem b in items)
          {
             ParentBarItem pbi = b as ParentBarItem;
             mergedItems.Add(pbi == null ? b : bm.GetMergedEquivalent(pbi, pbi));
          }

          foreach (BarItem b in bm.Items)
          {
             // ignore the "&" symbol
             if (mergedItems.Contains(b) && b.Text.Replace("&", "") == text.Replace("&", "")) return b;
          }
          return null;
       }

       private void barItem2_Click(object sender, EventArgs e)
       {
          Bar mergedMainMenu = mainMenu.Manager.GetMergedEquivalent(mainMenu, mainMenu);
          //ParentBarItem pbi = (ParentBarItem)FindByText1(mergedMainMenu.Manager, mergedMainMenu.Items, "View");
          ParentBarItem pbi = (ParentBarItem)FindByText(mainMenu.Items, "View");
          BarItem item = new BarItem("foo");
          pbi.Manager.Items.Add(item);
          pbi.Items.Add(item);
       }

       private void barItem1_Click(object sender, EventArgs e)
       {
          if (MessageBox.Show("Sure?", "Exit", MessageBoxButtons.YesNo) == DialogResult.Yes)
          {
             Close();
          }
       }

       private void biSaveProjectLayout_Click(object sender, EventArgs e)
       {
          MemoryStream layoutStream = new MemoryStream();
          AppStateSerializer serializer = new AppStateSerializer(SerializeMode.BinaryFmtStream, layoutStream);
          mdiManager.SaveTabGroupStates(serializer);
          serializer.PersistNow();
          layoutStream.Position = 0;
          string layout = Convert.ToBase64String(layoutStream.ToArray());

          XmlDocument doc = new XmlDocument();
          XmlElement projectNode = doc.CreateElement("project");
          XmlElement layoutNode = doc.CreateElement("projectLayout");
          layoutNode.InnerText = layout;
          projectNode.AppendChild(layoutNode);
          foreach (View view in mdiManager.MdiChildren)
          {
             view.SaveTo(projectNode);
          }
          doc.AppendChild(projectNode);
            
          doc.Save("project.xml");
       }

       private void biLoadProjectLayout_Click(object sender, EventArgs e)
       {
          CloseAll();

          XmlDocument doc = new XmlDocument();
          doc.Load("project.xml");

          foreach (XmlNode viewNode in doc.GetElementsByTagName("view"))
          {
             View view = new View();
             view.LoadFrom(viewNode);
             view.MdiParent = this;
             view.Show();
          }

          char[] projectLayout = doc["project"]["projectLayout"].InnerText.ToCharArray();
          MemoryStream ms = new MemoryStream(Convert.FromBase64CharArray(projectLayout, 0, projectLayout.Length));
          mdiManager.LoadTabGroupStates(new AppStateSerializer(SerializeMode.BinaryFmtStream, ms));
       }

       private void CloseAll()
       {
          foreach (Form form in mdiManager.MdiChildren)
          {
             form.Close();
          }
       }

       private void biSwitcher_Click(object sender, EventArgs e)
       {
          Form f = new Form();
          for (int i = 0; i < mdiManager.MdiChildren.Length; i++)
          {
             Form view = mdiManager.MdiChildren[i];
             Button b = new Button();
             b.Location = new Point(10, i * 30);
             b.Text = view.Name;
             b.Click += delegate { ActivateMdiChild(view); };
             f.Controls.Add(b);
          }
          f.ShowDialog(this);
       }

       private void biViewAddToolbox_Click(object sender, EventArgs e)
       {
          TestControl viewer = new TestControl();
          viewer.name = "Toolbox";
          dockingManager1.SetEnableDocking(viewer, true);
          dockingManager1.SetAutoHideButtonVisibility(viewer, false);
          dockingManager1.SetAutoHideMode(viewer, false);
          dockingManager1.SetDockLabel(viewer, "Toolbox");
          dockingManager1.SetMenuButtonVisibility(viewer, false);
          dockingManager1.SetAsMDIChild(viewer, false);
          dockingManager1.SetDockAbility(viewer, DockAbility.All ^ DockAbility.Tabbed);
          dockingManager1.DockContextMenu += delegate(object s, DockContextMenuEventArgs arg)
                                             {
                                                if (arg.Owner == viewer)
                                                {
                                                   arg.ContextMenu.ParentBarItem.Items[2].Visible = false;
                                                }
                                             };
       }
    }
}