using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Xml;
using Syncfusion.Runtime.Serialization;
using Syncfusion.Windows.Forms.Tools;

namespace Baby3dx
{
   public partial class View : Form
   {
      private Stack<Control> viewers = new Stack<Control>();
      private static int n;
      private static int viewCount;
      private Random rnd = new Random();

      public View()
      {
         InitializeComponent();
         n++;
         Text = Name = "View" + viewCount++;
         Icon = Icon.FromHandle(((Bitmap)imageList1.Images[0]).GetHicon());

         //dockingManager1.DockVisibilityChanged += delegate(object sender, DockVisibilityChangedEventArgs arg)
         //                                         {
         //                                            if (!arg.Control.Visible)
         //                                            {
         //                                               MessageBox.Show(arg.Control.Name + " is closed");
         //                                            }
         //                                         };
      }

      private void biViewAddViewer_Click(object sender, EventArgs e)
      {
         TestControl p = new TestControl();
         //RichTextBox p = new RichTextBox();

         p.Name = rnd.Next().ToString();
         viewers.Push(p);
         dockingManager1.SetEnableDocking(p, true);
         dockingManager1.FloatControl(p, new Rectangle(PointToScreen(new Point(0, 0)), new Size(100, 100)));
         //dockingManager1.FloatControl(p, new Rectangle(new Rectangle(PointToScreen(new Point(0, 0)), new Size(50, 100))));
      }

      private void biViewRemoveViewer_Click(object sender, EventArgs e)
      {
         if (viewers.Count == 0) return;
         Control p = viewers.Pop();
         dockingManager1.SetDockVisibility(p, false);
         //dockingManager1.SetEnableDocking(p, false);
      }

      protected override void OnClosed(EventArgs e)
      {
         base.OnClosed(e);
         if (--n > 1)
         {
            //MessageBox.Show("Note the incorrect state of the toolbars.");
         }
      }

      private void RemoveAllViewers()
      {
         foreach (Control c in dockingManager1.ControlsArray)
         {
            dockingManager1.SetEnableDocking(c, false);

            //Dispose() has to be called to avoid redrawing glitches
            //ATTENTION Syncfusion guys - in the real app, we can't dispose those controls!
            c.Dispose();
         }
      }

      public XmlElement GetViewerTypesXml(XmlDocument doc)
      {
         XmlElement types = doc.CreateElement("viewers");
         foreach (Control c in dockingManager1.ControlsArray)
         {
            XmlElement nameType = doc.CreateElement("viewer");
            nameType.SetAttribute("name", c.Name);
            types.AppendChild(nameType);
         }
         return types;
      }

      public void LoadFrom(XmlNode viewNode)
      {
         RemoveAllViewers();

         // construct viewers according to the viewer types
         foreach (XmlNode viewer in viewNode["viewers"].ChildNodes)
         {
            string name = viewer.Attributes["name"].Value;
            Control c = new TestControl();
            c.Name = name;
            dockingManager1.SetEnableDocking(c, true);
         }

         // apply layout
         char[] content = viewNode["layout"].InnerText.ToCharArray();
         MemoryStream ms = new MemoryStream(Convert.FromBase64CharArray(content, 0, content.Length));
         dockingManager1.LoadDockState(new AppStateSerializer(SerializeMode.BinaryFmtStream, ms));
      }

      public XmlElement GetLayoutXml(XmlDocument doc)
      {
         XmlElement layoutNode = doc.CreateElement("layout");
         MemoryStream layoutStream = new MemoryStream();

         AppStateSerializer serializer = new AppStateSerializer(SerializeMode.BinaryFmtStream, layoutStream);
         dockingManager1.SaveDockState(serializer);
         serializer.PersistNow();

         layoutStream.Position = 0;
         layoutNode.InnerXml = Convert.ToBase64String(layoutStream.ToArray());
         return layoutNode;
      }

      public void SaveTo(XmlNode node)
      {
         XmlDocument doc = node.OwnerDocument;
         XmlElement typedLayout = doc.CreateElement("view");
         typedLayout.AppendChild(GetViewerTypesXml(doc));
         typedLayout.AppendChild(GetLayoutXml(doc));
         node.AppendChild(typedLayout);
      }

      private void biViewSaveLayout_Click(object sender, EventArgs e)
      {
         XmlDocument doc = new XmlDocument();
         doc.AppendChild(doc.CreateElement("layout_only"));
         SaveTo(doc["layout_only"]);
         doc.Save("layout.xml");
      }

      private void biViewApplyLayout_Click(object sender, EventArgs e)
      {
         XmlDocument doc = new XmlDocument();
         doc.Load("layout.xml");
         LoadFrom(doc["layout_only"]["view"]);
      }

      private void biViewAddMainViewer_Click(object sender, EventArgs e)
      {
         Control c = new TestControl();
         c.Name = "Main";
         dockingManager1.SetEnableDocking(c, true);
         dockingManager1.SetDockLabel(c, "Main viewer");
         dockingManager1.DockToFill = true;
         dockingManager1.DockControl(c, this, DockingStyle.Bottom, 100);
      }

      private void biViewAddToolbar_Click(object sender, EventArgs e)
      {
         Control c = new TestControl();
         c.Name = "View toolbar";
         dockingManager1.SetEnableDocking(c, true);
         dockingManager1.SetDockLabel(c, null);
         dockingManager1.DockControl(c, this, DockingStyle.Top, 50);
         dockingManager1.SetOuterDockAbility(c, DockAbility.None);
         dockingManager1.SetDockAbility(c, DockAbility.None);
         dockingManager1.DragAllow += delegate(object s, DragAllowEventArgs arg) { arg.Cancel = (arg.Control == c); };
         dockingManager1.DockAllow += delegate(object s, DockAllowEventArgs arg) { arg.Cancel = (arg.DragControl == c); };

         DockHost dockhost = (DockHost)c.Parent;
         ((DockHostController)dockhost.InternalController).HideCaption = true;

         //dockingManager1.ShowCaption = false;
      }
   }
}